"use strict";
const graphql_1 = require("graphql");
/**
 * Creates a type-safe non-null enum GraphQL type.
 */
function createEnumType(config) {
    return new StrongGraphQLEnumType(new StrongGraphQLNullableEnumType(config));
}
exports.createEnumType = createEnumType;
/**
 * The non-null strong GraphQL enum type object.
 */
class StrongGraphQLEnumType extends graphql_1.GraphQLNonNull {
    constructor(nullableType) {
        super(nullableType);
        // The required type flags.
        this._strongType = true;
        this._strongInputType = true;
        this._strongOutputType = true;
        this._strongValue = null;
    }
    // The required type conversion methods.
    _weakType() { return this; }
    _weakInputType() { return this; }
    _weakOutputType() { return this; }
    /**
     * Returns the inner nullable variation of this type.
     */
    nullable() {
        return this.ofType;
    }
}
/**
 * The nullable sstrong GraphQL enum type object.
 */
class StrongGraphQLNullableEnumType extends graphql_1.GraphQLEnumType {
    constructor(config) {
        super(config);
        // The required type flags.
        this._strongType = true;
        this._strongInputType = true;
        this._strongOutputType = true;
        this._strongValue = null;
    }
    // The required type conversion methods.
    _weakType() { return this; }
    _weakInputType() { return this; }
    _weakOutputType() { return this; }
    /**
     * Returns self.
     */
    nullable() {
        return this;
    }
}
