"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
const graphql_1 = require("graphql");
function createObjectType(config) {
    return new StrongGraphQLObjectType(new StrongGraphQLNullableObjectType(config, []));
}
exports.createObjectType = createObjectType;
/**
 * The object returned by `createObjectType`. It is non-null, to get the
 * nullable variant just call `.nullable()`.
 */
// Developers could just instantiate this object directly instead of using
// `createObjectType`, but the function interface feels nicer and allows us to
// add extra features like function overloading.
class StrongGraphQLObjectType extends graphql_1.GraphQLNonNull {
    constructor(nullableType) {
        super(nullableType);
        // The required type flags.
        this._strongType = true;
        this._strongOutputType = true;
        this._strongValue = null;
        /**
         * A schema created for executing queries against where the query type is this
         * object type.
         */
        this._schema = null;
        this.name = nullableType.name;
    }
    // The required type conversion methods.
    _weakType() { return this; }
    _weakOutputType() { return this; }
    field(config) {
        return new StrongGraphQLObjectType(this.ofType._field(config));
    }
    fieldNonNull(config) {
        return new StrongGraphQLObjectType(this.ofType._fieldNonNull(config));
    }
    /**
     * Returns the inner nullable version of this type without mutating anything.
     */
    nullable() {
        return this.ofType;
    }
    /**
     * Executes a GraphQL query against this type. The schema used for executing
     * this query uses this object type as the query object type. There is no
     * mutation or subscription type.
     *
     * This can be very useful in testing.
     */
    execute(query, value, context, variables = {}, operation) {
        if (this._schema == null)
            this._schema = new graphql_1.GraphQLSchema({ query: this.ofType });
        return graphql_1.graphql(this._schema, query, value, context, variables, operation);
    }
}
exports.StrongGraphQLObjectType = StrongGraphQLObjectType;
/**
 * The private nullable implementation of `StrongGraphQLObjectType`. Because we
 * want types to be non-null by default, but in GraphQL types are nullable by
 * default this type is also the one that actually extends from
 * `GraphQLObjectType`.
 */
class StrongGraphQLNullableObjectType extends graphql_1.GraphQLObjectType {
    constructor(config, fieldConfigs) {
        super({
            name: config.name,
            description: config.description,
            isTypeOf: config.isTypeOf,
            // We define a thunk which computes our fields from the fields config
            // array we’ve built.
            fields: () => {
                const fields = {};
                for (const fieldConfig of fieldConfigs) {
                    // Create an args object that we will give to our field config. This
                    // arguments object will be mutated later and filled with argument
                    // configs.
                    const argsDefinition = {};
                    fields[fieldConfig.name] = {
                        description: fieldConfig.description,
                        deprecationReason: fieldConfig.deprecationReason,
                        type: typeof fieldConfig.type === 'function' ? fieldConfig.type()._weakOutputType() : fieldConfig.type._weakOutputType(),
                        args: argsDefinition,
                        resolve: (source, args, context) => fieldConfig.resolve(source, args, context),
                    };
                    // If the field has defined some arguments, loop through the arguments
                    // that exist and add them to the `args` object.
                    if (fieldConfig.args) {
                        for (const argName in fieldConfig.args) {
                            if (fieldConfig.args.hasOwnProperty(argName)) {
                                const argConfig = fieldConfig.args[argName];
                                argsDefinition[argName] = {
                                    type: argConfig.type._weakInputType(),
                                    defaultValue: argConfig.defaultValue,
                                    description: argConfig.description,
                                };
                            }
                        }
                    }
                }
                return fields;
            },
        });
        // The required type flags.
        this._strongType = true;
        this._strongOutputType = true;
        this._strongValue = null;
        this._strongConfig = config;
        this._strongFieldConfigs = fieldConfigs;
    }
    // The required type conversion methods.
    _weakType() { return this; }
    _weakOutputType() { return this; }
    /**
     * Returns true if we already have a field of this name.
     */
    _hasField(fieldName) {
        return Boolean(this._strongFieldConfigs.find(({ name }) => name === fieldName));
    }
    /**
     * Throws an error if we already have a field with the provided name,
     * otherwise the function does nothing.
     */
    _assertUniqueFieldName(fieldName) {
        if (this._hasField(fieldName))
            throw new Error(`Type '${this.name}' already has a field named '${fieldName}'.`);
    }
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _field(config) {
        this._assertUniqueFieldName(config.name);
        return new StrongGraphQLNullableObjectType(this._strongConfig, [...this._strongFieldConfigs, __assign({}, config, { type: () => typeof config.type === 'function' ? config.type().nullable() : config.type.nullable() })]);
    }
    /**
     * This field is a private implementation detail and should not be used
     * outside of `StrongGraphQLObjectType`!
     */
    _fieldNonNull(config) {
        this._assertUniqueFieldName(config.name);
        return new StrongGraphQLNullableObjectType(this._strongConfig, [...this._strongFieldConfigs, config]);
    }
    /**
     * Returns self.
     */
    nullable() {
        return this;
    }
}
exports.StrongGraphQLNullableObjectType = StrongGraphQLNullableObjectType;
