"use strict";
const graphql_1 = require("graphql");
function createSchema(config) {
    return new StrongGraphQLSchema(config);
}
exports.createSchema = createSchema;
/**
 * The strong GraphQL schema represents a type-safe GraphQL schema that forces
 * type correctness on execution with the `execute` method.
 */
class StrongGraphQLSchema extends graphql_1.GraphQLSchema {
    constructor(config) {
        super({
            query: config.query.ofType,
            mutation: config.mutation && config.mutation.ofType,
            subscription: config.subscription && config.subscription.ofType,
        });
    }
    /**
     * A type-safe execution function for our strong GraphQL schema. With this
     * function you will be forced to provide values and contexts of the correct
     * types.
     */
    execute(query, value, context, variables = {}, operation) {
        return graphql_1.graphql(this, query, value, context, variables, operation);
    }
}
exports.StrongGraphQLSchema = StrongGraphQLSchema;
